/*
 * $Id: OpenManyVPI.java,v 1.2 2004/12/14 00:28:35 norman Exp $
 *
 * Copyright (C) 2004 by Brockmann Consult (info@brockmann-consult.de)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation. This program is distributed in the hope it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.bc.beam.visat;

import org.esa.beam.framework.dataio.ProductIO;
import org.esa.beam.framework.datamodel.Product;
import org.esa.beam.framework.ui.command.CommandAdapter;
import org.esa.beam.framework.ui.command.CommandEvent;
import org.esa.beam.framework.ui.command.ExecCommand;
import org.esa.beam.framework.ui.task.ObservableTask;
import org.esa.beam.framework.ui.task.TaskMonitor;
import org.esa.beam.framework.ui.task.TaskObserver;
import org.esa.beam.framework.ui.UIUtils;
import org.esa.beam.util.Debug;
import org.esa.beam.util.PropertyMap;
import org.esa.beam.visat.VisatPlugIn;
import org.esa.beam.visat.VisatApp;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * This VISAT plugin can open many products of different type at the same time.
 * It plugs into the file menue of VISAT.
 *
 * @author Sabine Embacher
 */
public class OpenManyVPI implements VisatPlugIn {

    private static final String KEY_LAST_OPEN = "openMany.path.lastopen";
    private VisatApp _visatApp;

    public void initPlugIn(final VisatApp visatApp) {
        _visatApp = visatApp;
        final ExecCommand execCommand = _visatApp.getCommandManager().createExecCommand("openMany", new CommandAdapter() {
            public void actionPerformed(CommandEvent event) {
                final JFileChooser jFileChooser = new JFileChooser();
                final String propertyString = getPreferences().getPropertyString(KEY_LAST_OPEN);
                if (propertyString != null && propertyString.trim().length() > 0) {
                    jFileChooser.setCurrentDirectory(new File(propertyString));
                }
                jFileChooser.setMultiSelectionEnabled(true);
                if (jFileChooser.showOpenDialog(_visatApp.getMainFrame()) == JFileChooser.APPROVE_OPTION) {
                    getPreferences().setPropertyString(KEY_LAST_OPEN,
                                                       jFileChooser.getCurrentDirectory().getAbsolutePath());
                    final File[] selectedFiles = jFileChooser.getSelectedFiles();
                    tryOpenProducts(selectedFiles);
                }
            }
        });
        execCommand.setText("Open many..."); /*I18N*/
        execCommand.setParent("file");
        execCommand.setMnemonic('m');
        execCommand.setPlaceAfter("open");
        execCommand.setPlaceBefore("reopen");
        execCommand.setSmallIcon(UIUtils.loadImageIcon("icons/Open16.gif"));
        execCommand.setLargeIcon(UIUtils.loadImageIcon("icons/Open24.gif"));
    }

    private void tryOpenProducts(final File[] selectedFiles) {
        final List ioErrorList = new ArrayList();
        final List noProductList = new ArrayList();
        final ObservableTask observableTask = new ObservableTask("Opening Products...", 0, selectedFiles.length - 1) { /*I18N*/
            protected void run() {
                for (int i = 0; i < selectedFiles.length; i++) {
                    File selectedFile = selectedFiles[i];
                    Product product = null;
                    setProgress(i);
                    try {
                        product = ProductIO.readProduct(selectedFile, null);
                        if (product != null) {
                            _visatApp.addProduct(product);
                        } else {
                            noProductList.add(selectedFile);
                        }
                    } catch (IOException e) {
                        Debug.trace(e);
                        ioErrorList.add(selectedFile);
                    }
                }
            }
        };
        observableTask.addTaskObserver(new TaskObserver() {
            public void taskPropertyChanged(ObservableTask task, String propertyName) {
                if (task.isFinished()) {
                    if (ioErrorList.size() > 0 || noProductList.size() > 0) {
                        showErrorMessage(ioErrorList, noProductList);
                    }
                }
            }
        });

        new TaskMonitor(_visatApp.getMainFrame(), "Open Products", observableTask);/*I18N*/
        observableTask.start();
    }

    private void showErrorMessage(final List ioErrorList, final List noProductList) {
        final StringBuffer msg = new StringBuffer();
        if (ioErrorList.size() > 0) {
            msg.append("\nAn I/O error occured, while opening the following files:\n"); /*I18N*/
            for (int i = 0; i < ioErrorList.size(); i++) {
                msg.append("   " + ((File) ioErrorList.get(i)).toString() + "\n");
            }
        }
        if (noProductList.size() > 0) {
            msg.append("\nThe following files have an unknown format:\n"); /*I18N*/
            for (int i = 0; i < noProductList.size(); i++) {
                msg.append("   " + ((File) noProductList.get(i)).toString() + "\n");
            }
        }
        JOptionPane.showMessageDialog(_visatApp.getMainFrame(), msg);
    }

    public void updateComponentTreeUI() {
    }

    private PropertyMap getPreferences() {
        return _visatApp.getPreferences();
    }
}
